/* This file is part of the libopencm3 project.
 *
 * It was generated by the irq2nvic_h script.
 *
 * These definitions bend every interrupt handler that is defined CMSIS style
 * to the weak symbol exported by libopencm3.
 */

#define WWDG_IRQHandler wwdg_isr
#define PVD_IRQHandler pvd_isr
#define RTC_IRQHandler rtc_isr
#define FLASH_IRQHandler flash_isr
#define RCC_IRQHandler rcc_isr
#define EXTI0_1_IRQHandler exti0_1_isr
#define EXTI2_3_IRQHandler exti2_3_isr
#define EXTI4_15_IRQHandler exti4_15_isr
#define TSC_IRQHandler tsc_isr
#define DMA1_CHANNEL1_IRQHandler dma1_channel1_isr
#define DMA1_CHANNEL2_3_IRQHandler dma1_channel2_3_isr
#define DMA1_CHANNEL4_5_IRQHandler dma1_channel4_5_isr
#define ADC_COMP_IRQHandler adc_comp_isr
#define LPTIM1_IRQHandler lptim1_isr
#define USART4_5_IRQHandler usart4_5_isr
#define TIM2_IRQHandler tim2_isr
#define TIM3_IRQHandler tim3_isr
#define TIM6_DAC_IRQHandler tim6_dac_isr
#define TIM7_IRQHandler tim7_isr
#define RESERVED4_IRQHandler reserved4_isr
#define TIM21_IRQHandler tim21_isr
#define I2C3_IRQHandler i2c3_isr
#define TIM22_IRQHandler tim22_isr
#define I2C1_IRQHandler i2c1_isr
#define I2C2_IRQHandler i2c2_isr
#define SPI1_IRQHandler spi1_isr
#define SPI2_IRQHandler spi2_isr
#define USART1_IRQHandler usart1_isr
#define USART2_IRQHandler usart2_isr
#define LPUART1_AES_RNG_IRQHandler lpuart1_aes_rng_isr
#define LCD_IRQHandler lcd_isr
#define USB_IRQHandler usb_isr
